/* LibTomCrypt, modular cryptographic library -- Tom St Denis
 *
 * LibTomCrypt is a library that provides various cryptographic
 * algorithms in a highly modular and flexible manner.
 *
 * The library is free for all purposes without any express
 * guarantee it works.
 *
 * Tom St Denis, tomstdenis@gmail.com, http://libtomcrypt.org
 *
 * slightly modified by the author of TCTEMP
 */
#include <tomcrypt.h>

/** 
   @file pkcs_5_2.c
   PKCS #5, Algorithm #2, Tom St Denis
*/
#ifdef PKCS_5

/**
   Execute PKCS #5 v2
   @param password          The input password (or key)
   @param password_len      The length of the password (octets)
   @param salt              The salt (or nonce)
   @param salt_len          The length of the salt (octets)
   @param iteration_count   # of iterations desired for PKCS #5 v2 [read specs for more]
   @param hash_idx          The index of the hash desired
   @param out               [out] The destination for this algorithm
   @param outlen            [in/out] The max size and resulting size of the algorithm output
   @return CRYPT_OK if successful
*/
int tc_pkcs_5_alg2(const unsigned char *password, unsigned long password_len, 
                const unsigned char *salt,     unsigned long salt_len,
                int iteration_count,           const struct ltc_hash_descriptor *pdHash,
                unsigned char *out,            unsigned long *outlen,
				PPKCS5_2_BUFFER pBuffer)
{
   int err, itts;
   ulong32  blkno;
   unsigned long stored, left, x, y;

   LTC_ARGCHK(password != NULL);
   LTC_ARGCHK(salt     != NULL);
   LTC_ARGCHK(out      != NULL);
   LTC_ARGCHK(outlen   != NULL);
   LTC_ARGCHK(pdHash   != NULL);

#if MAX_PKCS5_2_BLOCK_SIZE != MAXBLOCKSIZE
#  error "Invalid block size!"
#endif

   left   = *outlen;
   blkno  = 1;
   stored = 0;
   while (left != 0) {
       /* process block number blkno */
       zeromem(pBuffer->buf0, MAXBLOCKSIZE*2);
       
       /* store current block number and increment for next pass */
       STORE32H(blkno, pBuffer->buf1);
       ++blkno;

       /* get PRF(P, S||int(blkno)) */
	   if ((err = tc_hmac_init(&pBuffer->hmac, pdHash, password, password_len)) != CRYPT_OK) { 
          goto LBL_ERR;
       }
       if ((err = hmac_process(&pBuffer->hmac, salt, salt_len)) != CRYPT_OK) {
          goto LBL_ERR;
       }
       if ((err = hmac_process(&pBuffer->hmac, pBuffer->buf1, 4)) != CRYPT_OK) {
          goto LBL_ERR;
       }
       x = MAXBLOCKSIZE;
       if ((err = hmac_done(&pBuffer->hmac, pBuffer->buf0, &x)) != CRYPT_OK) {
          goto LBL_ERR;
       }

       /* now compute repeated and XOR it in pBuffer->buf1 */
       XMEMCPY(pBuffer->buf1, pBuffer->buf0, x);
       for (itts = 1; itts < iteration_count; ++itts) {
		   if ((err = tc_hmac_memory(pdHash, password, password_len, pBuffer->buf0, x, pBuffer->buf0, &x, &pBuffer->hmac)) != CRYPT_OK) {
              goto LBL_ERR;
           }
           for (y = 0; y < x; y++) {
               pBuffer->buf1[y] ^= pBuffer->buf0[y];
           }
       }

       /* now emit upto x bytes of pBuffer->buf1 to output */
       for (y = 0; y < x && left != 0; ++y) {
           out[stored++] = pBuffer->buf1[y];
           --left;
       }
   }
   *outlen = stored;

   err = CRYPT_OK;
LBL_ERR:
   return err;
}

#endif


/* $Source: /cvs/libtom/libtomcrypt/src/misc/pkcs5/pkcs_5_2.c,v $ */
/* $Revision: 1.4 $ */
/* $Date: 2005/05/05 14:35:59 $ */
